#!/usr/bin/env python
# Time-stamp: <2017-06-13 12:51:00 Hongbo Liu>

"""Description: SMART main executable

Copyright (c) 2017 Hongbo Liu <hongbo919@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included
with the distribution).

@status: release candidate
@version: 2.1.8
@author:  Hongbo Liu
@contact: hongbo919@gmail.com
"""
# ------------------------------------
# python modules
# Add "SMART." before "DeNovoDMR_Calling import DeNovoDMR_Calling"
# dos2unix SMART
# ------------------------------------
import time
import os
import sys
import argparse
from DeNovoDMR_Calling import DeNovoDMR_Calling

def restricted_float_M(x):
    x = float(x)
    if x < 0.01 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.01, 1.0]"%(x,))
    return x

def restricted_float_G(x):
    x = float(x)
    if x < 0.5 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.5, 1.0]"%(x,))
    return x

def restricted_float_MSThreshold(x):
    x = float(x)
    if x < 0.2 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.2, 1.0]"%(x,))
    return x

def restricted_float_EDThreshold(x):
    x = float(x)
    if x < 0.2 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.01, 0.5]"%(x,))
    return x

def restricted_float_SMThreshold(x):
    x = float(x)
    if x < 0.2 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.01, 1.0]"%(x,))
    return x

def restricted_int_CDThreshold(x):
    x = int(x)
    if x < 1 or x > 2000:
        raise argparse.ArgumentTypeError("%r not in range [1, 2000]"%(x,))
    return x

def restricted_int_DLThreshold(x):
    x = int(x)
    if x < 1:
        raise argparse.ArgumentTypeError("%r should be larger than 1 bp"%(x,))
    return x

def restricted_int_pThreshold(x):
    x = float(x)
    if x < 1.0e-100 or x > 0.05:
        raise argparse.ArgumentTypeError("%r should be larger than 1 bp"%(x,))
    return x

# ------------------------------------
# Main function
# ------------------------------------
def main(argv):
    """The Main function/pipeline for SMART.
    """
    # Parse options...
    description = "%(prog)s 2.1 -- Specific Methylation Analysis and Report Tool for BS-Sequencing Data"
    epilog = "For any help, type: %(prog)s -h, or write to Hongbo Liu (hongbo919@gmail.com)"
    parser = argparse.ArgumentParser(description = description, epilog = epilog)
    parser.add_argument("MethylMatrix", type = str, default = '',
                        help = "The input methylation file (such as /WGBS/MethylMatrix.txt) including methylation values in all samples to compare (REQUIRED). The methylation data should be arranged as a matrix in which each row represents a CpG site. The columns are tab-separated. The column names should be included in the first line, with the first three columns representing the location of CpG sites: chrom, start, end. The methylation values starts from the fourth column. And the methylation value should be between 0 (unmethylated) to 1 (fully methylated). The missing values should be shown as -. The names of samples should be given as G1_1,G1_2,G2_1,G2_2,G3_1,G3_2,G3_3, in which Gi represents group i. \nThe Methylation matrix can be build based on bed files (chrom start end betavalue) by bedtools as: bedtools unionbedg -i G1_1.bed G1_2.bed G2_1.bed  G2_2.bed  G3_1.bed   G3_2.bed   G3_3.bed -header -names G1_1 G1_2 G2_1 G2_2 G3_1 G3_2 G3_3 -filler - > MethylMatrix.txt. The example data is also available at http://fame.edbc.org/smart/Example_Data_for_SMART2.zip. [Type: file]")
    parser.add_argument("-t", dest='ProjectType',type = str, choices=['DeNovoDMR','DMROI','DMC'], default = 'DeNovoDMR',
                        help = "Type of project including 'DeNovoDMR','DMROI' and 'DMC'. DeNovoDMR means de novo identification of differentially methylated regions (DMRs) based on genome segmentation. DMROI means the comparison of the methylation difference in regions of interest (ROIs) across multiple groups. DMC means identification of differentially methylated CpG sites (DMCs). DEFAULT: \'DeNovoDMR\' [Type: string]")
    parser.add_argument("-g", dest='GenomeRegions',type = str, default = '',
                        help = "Genome of regions of interest in bed format without column names (such as /WGBS/Regions_of_interest.bed) for project type DMROI. The regions in the file should be sorted by chromosome and then by start position (e.g., sort -k1,1 -k2,2n in.bed > in.sorted.bed). If this file is provided, SMART treat each region as a unit and compare its mean methylation across groups by methylation specificity and ANOVA analysis. DEFAULT: \'\' [Type: string]")
    parser.add_argument("-n", dest='ProjectName',type = str, default = "SMART",
                        help = "Project name, which will be used to generate output file names. DEFAULT: \"SMART\" [Type: string]")
    parser.add_argument("-o", dest='OutputFolder',type = str, default = '',
                        help = "The folder in which the result will be output. If specified all output files will be written to that directory. [Type: folder] [DEFAULT: the directory named using project name and current time (such as SMART20140801132559) in the current working directory]")
    parser.add_argument("-MR", dest='MissReplace',type = restricted_float_M, default = 0.5,
                        help = "Replace the missing value with the mediate methylation value of available samples in the corresponding group. The user can control whether to replace missing value by setting this parameter from 0.01 (meaning methylation values are available in at least 1%% of samples) to 1.0 (meaning methylation values are available in 100%% of samples, i.e there is no missing values). [Type: float] [Range: 0.01 ~ 1.0] [DEFAULT: 0.5]")
    parser.add_argument("-MS", dest='MSthreshold',type = restricted_float_MSThreshold, default = 0.5,
                        help = "Methylation Specificity Threshold for DMC or DMR calling. This parameter can be used to identify DMC or DMR as the the CpG site or region with methylation specificity which is greater than the threshold. [Type: float] [Range: 0.2 ~ 1.0] [DEFAULT: 0.5]")
    parser.add_argument("-ED", dest='EDthreshold',type = restricted_float_EDThreshold, default = 0.2,
                        help = "Euclidean Distance Threshold for methylation similarity between neighboring CpGs which is used in genome segmentation for de novo identification of DMR. The methylation similarity between neighboring CpGs is high if the Euclidean distance is less than the threshold. [Type: float] [Range: 0.01 ~ 0.5] [DEFAULT: 0.2]")
    parser.add_argument("-SM", dest='SMthreshold',type = restricted_float_SMThreshold, default = 0.6,
                        help = "Similarity Entropy Threshold for methylation similarity between neighboring CpGs which is used in genome segmentation for de novo identification of DMR. The methylation similarity between neighboring CpGs is high if similarity entropy is less than the threshold. [Type: float] [Range: 0.01 ~ 1.0] [DEFAULT: 0.6]")
    parser.add_argument("-CD", dest='CDthreshold',type = restricted_int_CDThreshold, default = 500,
                        help = "CpG Distance Threshold for the maximal distance between neighboring CpGs which is used in genome segmentation for de novo identification of DMR. The neighboring CpGs will be merged if the distance less than this threshold. [Type: int] [Range: 1 ~ 2000] [DEFAULT: 500]")
    parser.add_argument("-SC", dest='SCthreshold',type = restricted_int_DLThreshold, default = 5,
                        help = "Segment CpG Number Threshold for the minimal number of CpGs of merged segment and de novo identified DMR. The segments/DMRs with CpG number larger than this threshold will be output for further analysis. [Type: int] [Range: > 1] [DEFAULT: 5]")
    parser.add_argument("-SL", dest='SLthreshold',type = restricted_int_DLThreshold, default = 20,
                        help = "Segment Length Threshold for the minimal length of merged segment and de novo identified DMR. The segments/DMRs with a length larger than this threshold will be output for further analysis. [Type: int] [Range: > 1] [DEFAULT: 20]")
    parser.add_argument("-pD", dest='p_DMR',type = restricted_int_pThreshold, default = 0.05,
                        help = "p value of one-way analysis of variance (ANOVA) which is carried out for identification of DMRs across multiple groups. The segments with p value less than this threshold are identified as DMR. [Type: float] [Range: 1.0e-100 ~ 0.05] [DEFAULT: 0.05]")
    parser.add_argument("-pM", dest='p_MethylMark',type = restricted_int_pThreshold, default = 0.05,
                        help = "p value of one sample t-test which is carried out for identification of Methylation mark in a specific group based on the identified DMRs. The DMRs with p value less than this threshold is identified as group- specific methylation mark (Hyper methylation mark or Hypo methylation mark). [Type: float] [Range: 1.0e-100 ~ 0.05] [DEFAULT: 0.05]")
    parser.add_argument("-v", "--version",dest='version',action='version', version='SMART 2.1.8')
        
    UserArgs = parser.parse_args()
    MethyData=UserArgs.MethylMatrix
    ProjectType=UserArgs.ProjectType
    GenomeRegions=UserArgs.GenomeRegions
    ProjectName=UserArgs.ProjectName
    OutFolder=UserArgs.OutputFolder
    MissReplace=UserArgs.MissReplace
    MSthreshold=UserArgs.MSthreshold
    EDthreshold=UserArgs.EDthreshold
    SMthreshold=UserArgs.SMthreshold
    CDthreshold=UserArgs.CDthreshold
    SCthreshold=UserArgs.SCthreshold
    SLthreshold=UserArgs.SLthreshold
    p_DMR=UserArgs.p_DMR
    p_MethylMark=UserArgs.p_MethylMark
    UserArgslist = str(vars(UserArgs))
    
    if (MethyData==''):
        sys.exit( "[Error:] Methylation data is needed. Please re-run it giving the location of the methylation data.")
    if (MethyData):
        if not os.path.exists( MethyData ):
                sys.exit( "[Error:] Methylation data directory (%s) could not be found. Please re-run it giving the correct location of the methylation data." % MethyData )
                
        if (OutFolder==''):
            homedir = os.getcwd()
            Starttime=time.strftime('%Y%m%d%H%M%S',time.localtime(time.time()))
            OutFolder=homedir+"/"+ProjectName+Starttime+"/"
            #print OutFolder
        if OutFolder:
            # use a output directory to store SMART output
            if OutFolder[-1]!='/':
                OutFolder=OutFolder+'/'
            
            if not os.path.exists( OutFolder ):
                try:
                    os.makedirs( OutFolder )
                except:
                    sys.exit( "[Error:] Output directory (%s) could not be created. Terminating program." % OutFolder )            
        
                
        if ProjectType == 'DeNovoDMR':
            DeNovoDMR_Callingrun=DeNovoDMR_Calling()
            DeNovoDMR_Callingrun.runDeNovo(UserArgslist,MethyData,ProjectName,MissReplace,MSthreshold,EDthreshold,SMthreshold,CDthreshold,SCthreshold,SLthreshold,p_DMR,p_MethylMark,OutFolder)
        elif ProjectType == 'DMROI':
            if (GenomeRegions==''):
                sys.exit( "[Error:] Genome Regions is needed. Please re-run it giving the location of the bed file for Genome Regions.")
            if (GenomeRegions):
                if not os.path.exists( GenomeRegions ):
                        sys.exit( "[Error:] Genome Regions file (%s) could not be found. Please re-run it giving the correct location of the bed file for Genome Regions." % GenomeRegions )
            DeNovoDMR_Callingrun=DeNovoDMR_Calling()
            DeNovoDMR_Callingrun.runDMROI(UserArgslist,MethyData,ProjectName,MissReplace,GenomeRegions,MSthreshold,EDthreshold,SMthreshold,CDthreshold,SCthreshold,SLthreshold,p_DMR,p_MethylMark,OutFolder)    
            
        elif ProjectType == 'DMC':
            DeNovoDMR_Callingrun=DeNovoDMR_Calling()
            DeNovoDMR_Callingrun.runDMC(UserArgslist,MethyData,ProjectName,MissReplace,MSthreshold,EDthreshold,SMthreshold,CDthreshold,SCthreshold,SLthreshold,p_DMR,p_MethylMark,OutFolder)
        else:
            sys.exit( "[Error:] Project Type (%s) is not supported. Please re-run it giving the correct project type 'DeNovoDMR','DMROI' or 'DMC'. " % ProjectType )
        


if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python
'''
Created on 2013.11.11

@author: liuhongbo
'''
import gzip
import Folderprocess
import NewEntropy
import time
import scipy.stats
import scipy.spatial
import os
import sys
from math import sqrt

class MethyMergeEntropy():
    '''
    This module is used merge multiple profiles of the methylation in a chrome into an combined file
    100003153    100
    1000170    26
    1000190    65
    1000191    67
    1000198    74
    '''
    
    def __init__(self):
        '''
        Constructor
        '''
    def Mergehashnull(self,chrome,Chromedirpath,OutFolderName):
        '''
        Constructor
        Read the methylation data in bed.gz, and combind all values into a file, Null is -1
        '''
        Chromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb')
        Firstline=chrome
        #Build a hash        
        Methyhash={}
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        #print filenames
        column=0 
        for filename in filenames:
            Firstline=Firstline+"\t"+filename
            column=column+1
            methyfile=gzip.open(filename,'rb')
            methyfile.readline()
            filecontent=methyfile.readline()
            while filecontent:
                filecontent=filecontent.strip('\n')
                methyinfor=filecontent.split('\t')
                Methyhash[(methyinfor[0],column)]=methyinfor[1]   #key is the location of C and the column, value is the methylation level
                filecontent=methyfile.readline()
        Chromemethyfile.write(Firstline)
        #Obtain the keys
        MethyLocation=[]
        MethyKeys=Methyhash.keys()
        for i in range(0,len(MethyKeys)):
            MethyLocation.append(MethyKeys[i][0])
        MethyKeys=[]    #
        #Remove the repeat and Sort the location of C
        MethyLocation=list(set(MethyLocation))
        MethyLocation.sort(key=int)
        #Visit the hash and Output methy values
        for methyrow in range(0,len(MethyLocation)):
            methyline=MethyLocation[methyrow]
            for methycolumn in range(1,column+1):
                methyvalue=Methyhash.get((MethyLocation[methyrow],methycolumn),-1)
                methyline=methyline+"\t"+str(methyvalue)
            Chromemethyfile.write(methyline+"\n")
        #MethyLocation=[]
        #Methyhash={}

    def Mergehash(self,chrome,Chromedirpath,OutFolderName):
        '''
        Constructor
        Read the methylation data in bed.gz, and combind all values into a file without null -1
        '''
        Chromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb')
        Firstline=chrome
        #Build a hash        
        Methyhash={}
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        #print filenames
        column=0 
        for filename in filenames:
            Firstline=Firstline+"\t"+filename
            column=column+1
            methyfile=gzip.open(filename,'rb')
            methyfile.readline()
            filecontent=methyfile.readline()
            while filecontent:
                filecontent=filecontent.strip('\n')
                methyinfor=filecontent.split('\t')
                Methyhash[(methyinfor[0],column)]=methyinfor[1]   #key is the location of C and the column, value is the methylation level
                filecontent=methyfile.readline()
        Chromemethyfile.write(Firstline)
        #Obtain the keys
        MethyLocation=[]
        MethyKeys=Methyhash.keys()
        for i in range(0,len(MethyKeys)):
            MethyLocation.append(MethyKeys[i][0])
        MethyKeys=[]    #
        #Remove the repeat and Sort the location of C
        MethyLocation=list(set(MethyLocation))
        MethyLocation.sort(key=int)
        #Visit the hash and Output methy values
        for methyrow in range(0,len(MethyLocation)):
            methyline=MethyLocation[methyrow]
            for methycolumn in range(1,column+1):
                methyvalue=Methyhash.get((MethyLocation[methyrow],methycolumn),-1)
                methyline=methyline+"\t"+str(methyvalue)
            if methyline.find("-1")<0:
                Chromemethyfile.write(methyline+"\n")
        #MethyLocation=[]
        #Methyhash={}
        
        
    def Mergehashmean(self,chrome,Chromedirpath,OutFolderName):
        '''
        Constructor
        Read the methylation data in bed.gz, and compute the mean of repeat samples, then calculate the entropy, at last combind all values into a file without null -1
        '''
        Chromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb')
        Firstline=chrome          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        #Obtain the repeat samples and store them into a dictionary
        filehash={}
        Samplelist=[]
        for filename in filenames:
            fileinfor=filename.split('Rep')
            Sample=fileinfor[0].split('MethylRrbs')[1]
            value=filehash.get(Sample,'')
            if value=='':
                filehash[Sample]=[filename]
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
            else:
                filehash[Sample].append(filename)
        Chromemethyfile.write(Firstline+"\n")
        
        #Build a hash        
        Methyhash={}
        for filename in filenames:
            methyfile=gzip.open(filename,'rb')
            methyfile.readline()
            filecontent=methyfile.readline()
            while filecontent:
                filecontent=filecontent.strip('\n')
                methyinfor=filecontent.split('\t')
                Methyhash[(methyinfor[0],filename)]=methyinfor[1]   #key is the location of C and the column, value is the methylation level
                filecontent=methyfile.readline()
        
        #Obtain the keys
        MethyLocation=[]
        MethyKeys=Methyhash.keys()
        for i in range(0,len(MethyKeys)):
            MethyLocation.append(MethyKeys[i][0])
        MethyKeys=[]    #
        #Remove the repeat and Sort the location of C
        MethyLocation=list(set(MethyLocation))
        MethyLocation.sort(key=int)
        
        #Visit the hash and Output methy values
        for methyrow in range(0,len(MethyLocation)):
            methyline=MethyLocation[methyrow]
            for Sample in Samplelist:
                SampleRepFiles=filehash.get(Sample,-1)
                MethySum=0
                Count=0
                for SampleRepFile in SampleRepFiles:
                    methyvalue=int(Methyhash.get((MethyLocation[methyrow],SampleRepFile),-1))
                    if methyvalue>=0:
                        MethySum=MethySum+methyvalue
                        Count=Count+1
                        #print Count
                if Count>0: #If at least one of the repeat sample has methylation value, compute the mean methylation level and record it
                    methyline=methyline+"\t"+str(MethySum/Count)
                    #print methyline
                else:  #If there are no methylation values in all repeat samples, exit current loop and set the methyline is null
                    methyline=''
                    break
            if methyline!='':  #IF methyline is not null, out put it into file
                Chromemethyfile.write(methyline+"\n")
        MethyLocation=[]
        Methyhash={}
        
        
    def MergehashmeanEntropy(self,chrome,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Read the methylation data in bed.gz, and combind all values into a file without null -1
        '''
        Chromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb')
        Firstline=chrome          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        Methyentropy.MaxMethylationLevel=MaxMethyValue
        #Obtain the repeat samples and store them into a dictionary
        filehash={}
        Samplelist=[]
        for filename in filenames:
            fileinfor=filename.split('Rep')
            Sample=fileinfor[0].split('MethylRrbs')[1]
            value=filehash.get(Sample,'')
            if value=='':
                filehash[Sample]=[filename]
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
            else:
                filehash[Sample].append(filename)
        Chromemethyfile.write(Firstline+"\tEntropy\n")
        
        #Build a hash        
        Methyhash={}
        for filename in filenames:
            methyfile=gzip.open(filename,'rb')
            methyfile.readline()
            filecontent=methyfile.readline()
            while filecontent:
                filecontent=filecontent.strip('\n')
                methyinfor=filecontent.split('\t')
                Methyhash[(methyinfor[0],filename)]=methyinfor[1]   #key is the location of C and the column, value is the methylation level
                filecontent=methyfile.readline()
        
        #Obtain the keys
        MethyLocation=[]
        MethyKeys=Methyhash.keys()
        for i in range(0,len(MethyKeys)):
            MethyLocation.append(MethyKeys[i][0])
        MethyKeys=[]    #
        #Remove the repeat and Sort the location of C
        MethyLocation=list(set(MethyLocation))
        MethyLocation.sort(key=int)
        
        #Visit the hash and Output methy values
        for methyrow in range(0,len(MethyLocation)):
            methyline=MethyLocation[methyrow]
            MethydataSet=[]
            for Sample in Samplelist:
                SampleRepFiles=filehash.get(Sample,-1)
                MethySum=0
                Count=0
                for SampleRepFile in SampleRepFiles:
                    methyvalue=int(Methyhash.get((MethyLocation[methyrow],SampleRepFile),-1))
                    if methyvalue>=0:
                        MethySum=MethySum+methyvalue
                        Count=Count+1
                        #print Count
                if Count>0: #If at least one of the repeat sample has methylation value, compute the mean methylation level and record it
                    Methymeanvalue=MethySum/Count
                    MethydataSet.append(Methymeanvalue)
                    methyline=methyline+"\t"+str(Methymeanvalue)
                    #print methyline
                else:  #If there are no methylation values in all repeat samples, exit current loop and set the methyline is null
                    methyline=''
                    break
            if methyline!='':  #IF methyline is not null, out put it into file
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=round(Methyentropy.EntropyCalculate(MethydataSet),3)
                methyline=methyline+"\t"+str(Entropyvalue)+"\n"
                Chromemethyfile.write(methyline)
                #print MethyLocation[methyrow]+MethydataSet+str(Methyentropy.EntropyCalculate(MethydataSet[1:]))
        MethyLocation=[]
        Methyhash={}
        
    def MergehashBSSeqWigGzEntropy(self,chrome,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Based on hash, need more Memory
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        '''
        Chromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb')
        Firstline=chrome          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        Methyentropy.MaxMethylationLevel=MaxMethyValue
        #Obtain the repeat samples and store them into a dictionary
        filehash={}
        Samplelist=[]
        for filename in filenames:
            fileinfor=filename.split('.Bisulfite-Seq.')
            Sample=fileinfor[0].split('UCSD.')[1]
            value=filehash.get(Sample,'')
            if value=='':
                filehash[Sample]=[filename]
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
            else:
                filehash[Sample].append(filename)
        Chromemethyfile.write(Firstline+"\tEntropy\n")
        
        #Build a hash        
        Methyhash={}
        for filename in filenames:
            methyfile=gzip.open(filename,'rb')
            methyfile.readline()
            filecontent=methyfile.readline()
            while filecontent:
                filecontent=filecontent.strip('\n')
                methyinfor=filecontent.split('\t')
                Methyhash[(methyinfor[0],filename)]=methyinfor[1]   #key is the location of C and the column, value is the methylation level
                filecontent=methyfile.readline()
        
        #Obtain the keys
        MethyLocation=[]
        MethyKeys=Methyhash.keys()
        for i in range(0,len(MethyKeys)):
            MethyLocation.append(MethyKeys[i][0])
        MethyKeys=[]    #
        #Remove the repeat and Sort the location of C
        MethyLocation=list(set(MethyLocation))
        MethyLocation.sort(key=int)
        
        #Visit the hash and Output methy values
        for methyrow in range(0,len(MethyLocation)):
            methyline=MethyLocation[methyrow]
            MethydataSet=[]
            for Sample in Samplelist:
                SampleRepFiles=filehash.get(Sample,-1)
                MethySum=0
                Count=0
                for SampleRepFile in SampleRepFiles:
                    methyvalue=int(Methyhash.get((MethyLocation[methyrow],SampleRepFile),-1))
                    if methyvalue>=0:
                        MethySum=MethySum+methyvalue
                        Count=Count+1
                        #print Count
                if Count>0: #If at least one of the repeat sample has methylation value, compute the mean methylation level and record it
                    Methymeanvalue=MethySum/Count
                    MethydataSet.append(Methymeanvalue)
                    methyline=methyline+"\t"+str(Methymeanvalue)
                    #print methyline
                else:  #If there are no methylation values in all repeat samples, exit current loop and set the methyline is null
                    methyline=''
                    break
            if methyline!='':  #IF methyline is not null, out put it into file
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=round(Methyentropy.EntropyCalculate(MethydataSet),3)
                methyline=methyline+"\t"+str(Entropyvalue)+"\n"
                Chromemethyfile.write(methyline)
                #print MethyLocation[methyrow]+MethydataSet+str(Methyentropy.EntropyCalculate(MethydataSet[1:]))
        MethyLocation=[]
        Methyhash={}
        
    def MergefileBSSeqWigGznoRepEntropybase0(self,chrome,ChromeCfileName,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Based on files, only need file input and output
        C file is 0 based
        Methylation data is 0 based
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        '''
        ChromeCfile=gzip.open(ChromeCfileName,'rb')   #open the C file of the chrome
        MergedChromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb') #open the output file
        Firstline=ChromeCfile.readline().strip('\n')          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        Methyentropy.MaxMethylationLevel=MaxMethyValue
        #Open the methy flies and store file handles into a dictionary; Obtain the sample names and store them into a list
        methyfilehandles={}
        CurrentSampleline={}
        Samplelist=[]
        CNum=0
        for filename in filenames:
            #Obtain the sample names and store them into a list
            fileinfor=filename.split('.Bisulfite-Seq.')
            Sample=fileinfor[0].split('UCSD.')[1]
            value=methyfilehandles.get(Sample,'')
            if value=='':
                chromemethyfile=gzip.open(filename,'rb')
                methyfilehandles[Sample]=chromemethyfile  #build a dictionary of methy files: key is sample, value is the file Handle
                CurrentSampleline[Sample]=chromemethyfile.readline()  #build a dictionary of methy files: key is sample, value is the file Handle
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
        MergedChromemethyfile.write(Firstline+"\tEntropy\n")
        
        #Merge the methy files
        CurrentCline=ChromeCfile.readline()
        PreviousC=0
        while CurrentCline:
            methyline=CurrentCline.strip('\n')
            print methyline
            CurrentCinfor=methyline.split('\t')
            CurrentC=int(CurrentCinfor[0])
            if (CurrentC-PreviousC)>10000000:
                print CurrentC
                PreviousC=CurrentC
            MethydataSet=[]
            SampleNum=0
            while SampleNum<len(Samplelist) and methyline!='':
                Sample=Samplelist[SampleNum]
                chromemethyfile=methyfilehandles.get(Sample,-1) #Obtain the sample file and compare their location
                filecontent=CurrentSampleline.get(Sample,'over')
                if filecontent=='':   #If filecontent=='', at least one methyfile has been over.
                    CurrentCline=False
                    break
                methyinfor=filecontent.strip('\n').split('\t')
                if int(methyinfor[0])==CurrentC:  #If current C is equal, give it to output line and MethydataSet[]
                    methyvalue=round(float(methyinfor[1]),3)
                    methyline=methyline+"\t"+str(methyvalue)
                    MethydataSet.append(methyvalue)
                    CurrentSampleline[Sample]=chromemethyfile.readline()
                elif int(methyinfor[0])<CurrentC:  #If current C is larger, go on in the methy file
                    Currentfileline=chromemethyfile.readline()
                    while Currentfileline:
                        methyinfor=Currentfileline.strip('\n').split('\t')
                        if int(methyinfor[0])<CurrentC:
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                        elif int(methyinfor[0])==CurrentC:
                            methyvalue=round(float(methyinfor[1]),3)
                            methyline=methyline+"\t"+str(methyvalue)
                            MethydataSet.append(methyvalue)
                            CurrentSampleline[Sample]=chromemethyfile.readline()
                            Currentfileline=False
                        else:
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                elif int(methyinfor[0])>CurrentC:  #If current C is smaller, go on in the C file
                    CurrentCline=ChromeCfile.readline()
                    while CurrentCline:
                        CurrentCinfor=CurrentCline.split('\t')
                        if int(methyinfor[0])<=int(CurrentCinfor[0]):
                            methyline=''
                            break
                        else:
                            CurrentCline=ChromeCfile.readline()                
                SampleNum+=1
            if len(MethydataSet)==len(Samplelist):  #IF methyline is not null, out put it into file
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=round(Methyentropy.EntropyCalculate(MethydataSet),3)
                methyline=methyline+"\t"+str(Entropyvalue)+"\n"
                MergedChromemethyfile.write(methyline)
                CNum+=1
        ChromeCfile.close()
        MergedChromemethyfile.close()
        for Sample in Samplelist: methyfilehandles.get(Sample,-1).close()
        print chrome+"\tC Num is\t"+str(CNum)
        
        
    def MergefileBSSeqWigGznoRepEntropyCbase0Methybase1(self,chrome,ChromeCfileName,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Based on files, only need file input and output
        C file is 0 based
        Methylation data is 1 based
        output is 1 based
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        '''
        ChromeCfile=gzip.open(ChromeCfileName,'rb')   #open the C file of the chrome
        MergedChromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb') #open the output file
        Firstline=ChromeCfile.readline().strip('\n')          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        Methyentropy.MaxMethylationLevel=MaxMethyValue
        #Open the methy flies and store file handles into a dictionary; Obtain the sample names and store them into a list
        methyfilehandles={}
        CurrentSampleline={}
        Samplelist=[]
        CNum=0
        for filename in filenames:
            #Obtain the sample names and store them into a list
            fileinfor=filename.split('.Bisulfite-Seq.')
            Sample=fileinfor[0].split('UCSD.')[1]
            value=methyfilehandles.get(Sample,'')
            if value=='':
                chromemethyfile=gzip.open(filename,'rb')
                methyfilehandles[Sample]=chromemethyfile  #build a dictionary of methy files: key is sample, value is the file Handle
                CurrentSampleline[Sample]=chromemethyfile.readline()  #build a dictionary of methy files: key is sample, value is the file Handle
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
        MergedChromemethyfile.write(Firstline+"\tEntropy\n")
        
        #Merge the methy files
        CurrentCline=ChromeCfile.readline()
        PreviousC=0
        while CurrentCline:
            CurrentCline=CurrentCline.strip('\n')
            CurrentCinfor=CurrentCline.split('\t')
            CurrentC=int(CurrentCinfor[0])+1
            if (CurrentC-PreviousC)>10000000:
                print CurrentC
                PreviousC=CurrentC
            methyline=CurrentCline.replace(CurrentCinfor[0],str(CurrentC))
            #print methyline
            MethydataSet=[]
            SampleNum=0
            while SampleNum<len(Samplelist) and methyline!='':
                Sample=Samplelist[SampleNum]
                SampleNum+=1
                chromemethyfile=methyfilehandles.get(Sample,-1) #Obtain the sample file and compare their location
                filecontent=CurrentSampleline.get(Sample,'over')
                if filecontent=='' or filecontent=="over" or len(filecontent)<=3: #a methyfile is over
                    CurrentCline=False
                    break
                methyinfor=filecontent.strip('\n').split('\t')
                if int(methyinfor[0])==CurrentC:  #If current C is equal, give it to output line and MethydataSet[]
                    methyvalue=round(float(methyinfor[1]),3)
                    methyline=methyline+"\t"+str(methyvalue)
                    MethydataSet.append(methyvalue)
                    Currentfileline=chromemethyfile.readline()
                    CurrentSampleline[Sample]=Currentfileline
                elif int(methyinfor[0])<CurrentC:  #If current C is larger, go on in the methy file
                    Currentfileline=chromemethyfile.readline()
                    while Currentfileline:
                        methyinfor=Currentfileline.strip('\n').split('\t')
                        if int(methyinfor[0])<CurrentC:
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                        elif int(methyinfor[0])==CurrentC:
                            methyvalue=round(float(methyinfor[1]),3)
                            methyline=methyline+"\t"+str(methyvalue)
                            MethydataSet.append(methyvalue)
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                        elif len(Currentfileline)>3:
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                elif int(methyinfor[0])>CurrentC:  #If current C is smaller, go on in the C file
                    CurrentCline=ChromeCfile.readline()
                    while CurrentCline:
                        CurrentCinfor=CurrentCline.split('\t')
                        CurrentC=int(CurrentCinfor[0])+1
                        if int(methyinfor[0])<=CurrentC:
                            methyline=''
                            break
                        else:
                            CurrentCline=ChromeCfile.readline()                
            if len(MethydataSet)==len(Samplelist):  #IF methyline is not null, out put it into file
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=round(Methyentropy.EntropyCalculate(MethydataSet),3)
                methyline=methyline+"\t"+str(Entropyvalue)+"\n"
                MergedChromemethyfile.write(methyline)
                CNum+=1
        ChromeCfile.close()
        MergedChromemethyfile.close()
        for Sample in Samplelist: methyfilehandles.get(Sample,-1).close()
        print chrome+"\tC Num is\t"+str(CNum)
        
    def MergefileBSSeqWigGznoRepEntropyCbase1Methybase1(self,chrome,ChromeCfileName,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Based on files, only need file input and output
        C file is 1 based
        Methylation data is 1 based
        output is 1 based
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        '''
        ChromeCfile=gzip.open(ChromeCfileName,'rb')   #open the C file of the chrome
        MergedChromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb') #open the output file
        EntropyWig=gzip.open(MergedmethyFolderName+"EntropyNWIE.wig.gz",'a') #Open a wig file 
        EntropyWig.write("variableStep chrom="+chrome+"\n")
        Firstline=ChromeCfile.readline().strip('\n')          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        #Open the methy flies and store file handles into a dictionary; Obtain the sample names and store them into a list
        methyfilehandles={}
        CurrentSampleline={}
        Samplelist=[]
        CNum=0
        for filename in filenames:
            #Obtain the sample names and store them into a list
            fileinfor=filename.split('.Bisulfite-Seq.')
            Sample=fileinfor[0].split('GSM')[1]
            value=methyfilehandles.get(Sample,'')
            if value=='':
                chromemethyfile=gzip.open(filename,'rb')
                methyfilehandles[Sample]=chromemethyfile  #build a dictionary of methy files: key is sample, value is the file Handle
                CurrentSampleline[Sample]=chromemethyfile.readline()  #build a dictionary of methy files: key is sample, value is the file Handle
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
        MergedChromemethyfile.write(Firstline+"\tEntropy\n")
        
        #Merge the methy files
        CurrentCline=ChromeCfile.readline()
        PreviousC=0
        while CurrentCline:
            CurrentCline=CurrentCline.strip('\n')
            CurrentCinfor=CurrentCline.split('\t')
            CurrentC=int(CurrentCinfor[0])
            if (CurrentC-PreviousC)>10000000:
                print CurrentC
                PreviousC=CurrentC
            methyline=CurrentCline.replace(CurrentCinfor[0],str(CurrentC))
            #print methyline
            MethydataSet=[]
            SampleNum=0
            while SampleNum<len(Samplelist) and methyline!='':
                Sample=Samplelist[SampleNum]
                SampleNum+=1
                chromemethyfile=methyfilehandles.get(Sample,-1) #Obtain the sample file and compare their location
                filecontent=CurrentSampleline.get(Sample,'over')
                if filecontent=='' or filecontent=="over" or len(filecontent)<=3: #a methyfile is over
                    CurrentCline=False
                    break
                methyinfor=filecontent.strip('\n').split('\t')
                if int(methyinfor[0])==CurrentC:  #If current C is equal, give it to output line and MethydataSet[]
                    methyvalue=round(float(methyinfor[1]),3)
                    methyline=methyline+"\t"+str(methyvalue)
                    MethydataSet.append(methyvalue)
                    Currentfileline=chromemethyfile.readline()
                    CurrentSampleline[Sample]=Currentfileline
                elif int(methyinfor[0])<CurrentC:  #If current C is larger, go on in the methy file
                    Currentfileline=chromemethyfile.readline()
                    while Currentfileline:
                        methyinfor=Currentfileline.strip('\n').split('\t')
                        if int(methyinfor[0])<CurrentC:
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                        elif int(methyinfor[0])==CurrentC:
                            methyvalue=round(float(methyinfor[1]),3)
                            methyline=methyline+"\t"+str(methyvalue)
                            MethydataSet.append(methyvalue)
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                        elif len(Currentfileline)>3:
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                elif int(methyinfor[0])>CurrentC:  #If current C is smaller, go on in the C file
                    CurrentCline=ChromeCfile.readline()
                    while CurrentCline:
                        CurrentCinfor=CurrentCline.split('\t')
                        CurrentC=int(CurrentCinfor[0])
                        if int(methyinfor[0])<=CurrentC:
                            methyline=''
                            break
                        else:
                            CurrentCline=ChromeCfile.readline()                
            if len(MethydataSet)==len(Samplelist):  #IF methyline is not null, out put it into file
                for i in range(len(MethydataSet)):
                    MethydataSet[i]=MethydataSet[i]*100.0/MaxMethyValue
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=Methyentropy.EntropyCalculate(MethydataSet)
                methyline=methyline+"\t"+str(Entropyvalue)+"\n"
                MergedChromemethyfile.write(methyline)
                EntropyWig.write(str(CurrentC)+"\t"+str(Entropyvalue)+"\n")
                CNum+=1
        ChromeCfile.close()
        MergedChromemethyfile.close()
        for Sample in Samplelist: methyfilehandles.get(Sample,-1).close()
        statisticout=open(MergedmethyFolderName+"statistic.txt",'a')
        print chrome+"\tC Num is\t"+str(CNum)
        statisticout.write(chrome+"\tC Num is\t"+str(CNum)+"\n")
        statisticout.close()
        EntropyWig.close()
   
    def MergefileBSSeqWigGznoRepEntropyCbase1Methybase1PCC(self,chrome,ChromeCfileName,Chromedirpath,OutFolderName,MaxMethyValue):
        '''
        Constructor
        Based on files, only need file input and output
        C file is 1 based
        Methylation data is 1 based
        output is 1 based
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        Caculate the pcc between neighbring CpG
        '''
        ChromeCfile=gzip.open(ChromeCfileName,'rb')   #open the C file of the chrome
        MergedChromemethyfile=gzip.open(OutFolderName+chrome+".txt.gz",'wb') #open the output file
        EntropyWig=gzip.open(MergedmethyFolderName+"EntropyNWIE.wig.gz",'a') #Open a wig file 
        EntropyWig.write("variableStep chrom="+chrome+"\n")
        Firstline=ChromeCfile.readline().strip('\n')          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        #Open the methy flies and store file handles into a dictionary; Obtain the sample names and store them into a list
        methyfilehandles={}
        CurrentSampleline={}
        Samplelist=[]
        CNum=0
        for filename in filenames:
            #Obtain the sample names and store them into a list
            fileinfor=filename.split('.Bisulfite-Seq.')
            Sample=fileinfor[0].split('GSM')[1]
            value=methyfilehandles.get(Sample,'')
            if value=='':
                chromemethyfile=gzip.open(filename,'rb')
                methyfilehandles[Sample]=chromemethyfile  #build a dictionary of methy files: key is sample, value is the file Handle
                CurrentSampleline[Sample]=chromemethyfile.readline()  #build a dictionary of methy files: key is sample, value is the file Handle
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
        MergedChromemethyfile.write(Firstline+"\tEntropy\tPCC\tpvalue\n")
        
        #Merge the methy files #calculate the entropy and pcc
        CurrentCline=ChromeCfile.readline()
        PreviousC=0
        while CurrentCline:
            CurrentCline=CurrentCline.strip('\n')
            CurrentCinfor=CurrentCline.split('\t')
            CurrentC=int(CurrentCinfor[0])
            if (CurrentC-PreviousC)>10000000:
                print CurrentC
                PreviousC=CurrentC
            methyline=CurrentCline.replace(CurrentCinfor[0],str(CurrentC))
            #print methyline
            MethydataSet=[]
            SampleNum=0
            while SampleNum<len(Samplelist) and methyline!='':
                Sample=Samplelist[SampleNum]
                SampleNum+=1
                chromemethyfile=methyfilehandles.get(Sample,-1) #Obtain the sample file and compare their location
                filecontent=CurrentSampleline.get(Sample,'over')
                if filecontent=='' or filecontent=="over" or len(filecontent)<=3: #a methyfile is over
                    CurrentCline=False
                    break
                methyinfor=filecontent.strip('\n').split('\t')
                if int(methyinfor[0])==CurrentC:  #If current C is equal, give it to output line and MethydataSet[]
                    methyvalue=round(float(methyinfor[1]),3)
                    methyline=methyline+"\t"+str(methyvalue)
                    MethydataSet.append(methyvalue)
                    Currentfileline=chromemethyfile.readline()
                    CurrentSampleline[Sample]=Currentfileline
                elif int(methyinfor[0])<CurrentC:  #If current C is larger, go on in the methy file
                    Currentfileline=chromemethyfile.readline()
                    while Currentfileline:
                        methyinfor=Currentfileline.strip('\n').split('\t')
                        if int(methyinfor[0])<CurrentC:
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                        elif int(methyinfor[0])==CurrentC:
                            methyvalue=round(float(methyinfor[1]),3)
                            methyline=methyline+"\t"+str(methyvalue)
                            MethydataSet.append(methyvalue)
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                        elif len(Currentfileline)>3:
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                elif int(methyinfor[0])>CurrentC:  #If current C is smaller, go on in the C file
                    CurrentCline=ChromeCfile.readline()
                    while CurrentCline:
                        CurrentCinfor=CurrentCline.split('\t')
                        CurrentC=int(CurrentCinfor[0])
                        if int(methyinfor[0])<=CurrentC:
                            methyline=''
                            break
                        else:
                            CurrentCline=ChromeCfile.readline()                
            if len(MethydataSet)==len(Samplelist):  #IF methyline is not null, calculate the entropy and pcc, out put them into file
                for i in range(len(MethydataSet)):
                    MethydataSet[i]=MethydataSet[i]*100.0/MaxMethyValue
                if CNum==0:
                    PreviousMethydataSet=MethydataSet
                PCC=scipy.stats.pearsonr(PreviousMethydataSet, MethydataSet)   #calculate the PCC between the methylation leveles of current C and previous C
                PreviousMethydataSet=MethydataSet
                MethydataSet=tuple(MethydataSet)
                Entropyvalue=Methyentropy.EntropyCalculate(MethydataSet)
                methyline=methyline+"\t"+str(Entropyvalue)+"\t"+str(round(PCC[0],3))+"\t"+str('%.2e'%PCC[1])+"\n"
                MergedChromemethyfile.write(methyline)
                EntropyWig.write(str(CurrentC)+"\t"+str(Entropyvalue)+"\n")
                CNum+=1
        ChromeCfile.close()
        MergedChromemethyfile.close()
        for Sample in Samplelist: methyfilehandles.get(Sample,-1).close()
        statisticout=open(MergedmethyFolderName+"statistic.txt",'a')
        print chrome+"\tC Num is\t"+str(CNum)
        statisticout.write(chrome+"\tC Num is\t"+str(CNum)+"\n")
        statisticout.close()
        EntropyWig.close()

    def MergefileBSSeqWigGznoRepEntropyCbase1Methybase1DiffEntropyED(self,chrome,ChromeCfileName,Chromedirpath,MergedmethyFolderName,MaxMethyValue,statisticout):
        '''
        Constructor
        Based on files, only need file input and output
        C file is 1 based
        Methylation data is 1 based
        output is 1 based
        Read the methylation data in wig.gz, and combind all values into a file without null -1
        Caculate the DifferEntropy and Euclidean Distance between neighbring CpG
        '''
        ChromeCfile=gzip.open(ChromeCfileName,'rb')   #open the C file of the chrome
        MergedChromemethyfile=gzip.open(MergedmethyFolderName+chrome+".txt.gz",'wb') #open the output file
        EntropyWig=gzip.open(MergedmethyFolderName+"MethylationSpecificity.wig.gz",'a') #Open a wig file 
        EntropyWig.write("variableStep chrom="+chrome+"\n")
        Firstline=ChromeCfile.readline().strip('\n')          
        MethyFolder=Folderprocess.Folderprocess()
        filenames=MethyFolder.readFolderfile(Chromedirpath)
        Methyentropy=NewEntropy.Entropy()
        #Open the methy flies and store file handles into a dictionary; Obtain the sample names and store them into a list
        methyfilehandles={}
        CurrentSampleline={}
        Samplelist=[]
        CNum=0
        TCNum=0
        CommonCNum=0
        for filename in filenames:
            #Obtain the sample names and store them into a list
            fileinfor=filename.split(chrome+'_')
            Sample=fileinfor[1].split('.wig')[0]
            value=methyfilehandles.get(Sample,'')
            if value=='':
                chromemethyfile=gzip.open(filename,'rb')
                methyfilehandles[Sample]=chromemethyfile  #build a dictionary of methy files: key is sample, value is the file Handle
                CurrentSampleline[Sample]=chromemethyfile.readline()  #build a dictionary of methy files: key is sample, value is the file Handle
                Samplelist.append(Sample)
                Firstline=Firstline+"\t"+Sample
        MergedChromemethyfile.write(Firstline+"\tEntropy\tDifferEntropy\tEuclideanDistance\n")
        SampleTotalNum=len(Samplelist)
        SqrtSampleTotalNum=sqrt(SampleTotalNum)
        #Merge the methy files #calculate the entropy and pcc
        CurrentCline=ChromeCfile.readline()
        TCNum=TCNum+1
        PreviousC=0
        while CurrentCline:
            CurrentCline=CurrentCline.strip('\n')
            CurrentCinfor=CurrentCline.split('\t')
            CurrentC=int(CurrentCinfor[0])
            if (CurrentC-PreviousC)>10000000:
                Starttime=time.strftime('%Y-%m-%d %X',time.localtime(time.time()))
                print Starttime+" Have finished "+str(CurrentC)+" C..."
                PreviousC=CurrentC
            methyline=CurrentCline.replace(CurrentCinfor[0],str(CurrentC))
            #print methyline
            MethydataSet=[]
            SampleNum=0
            while SampleNum<SampleTotalNum and methyline!='':
                Sample=Samplelist[SampleNum]
                SampleNum+=1
                chromemethyfile=methyfilehandles.get(Sample,-1) #Obtain the sample file and compare their location
                filecontent=CurrentSampleline.get(Sample,'over')
                if filecontent=='' or filecontent=="over" or len(filecontent)<=3: #a methyfile is over
                    CurrentCline=False
                    break
                methyinfor=filecontent.strip('\n').split('\t')
                if int(methyinfor[0])==CurrentC:  #If current C is equal, give it to output line and MethydataSet[]
                    methyvalue=round(float(methyinfor[1]),3)
                    methyline=methyline+"\t"+str(methyvalue)
                    MethydataSet.append(methyvalue)
                    Currentfileline=chromemethyfile.readline()
                    CurrentSampleline[Sample]=Currentfileline
                elif int(methyinfor[0])<CurrentC:  #If current C is larger, go on in the methy file
                    Currentfileline=chromemethyfile.readline()
                    while Currentfileline:
                        methyinfor=Currentfileline.strip('\n').split('\t')
                        if int(methyinfor[0])<CurrentC:
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                        elif int(methyinfor[0])==CurrentC:
                            methyvalue=round(float(methyinfor[1]),3)
                            methyline=methyline+"\t"+str(methyvalue)
                            MethydataSet.append(methyvalue)
                            Currentfileline=chromemethyfile.readline()
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                        elif len(Currentfileline)>3:
                            CurrentSampleline[Sample]=Currentfileline
                            Currentfileline=False
                            break
                elif int(methyinfor[0])>CurrentC:  #If current C is smaller, go on in the C file
                    CurrentCline=ChromeCfile.readline()
                    TCNum=TCNum+1
                    while CurrentCline:
                        CurrentCinfor=CurrentCline.split('\t')
                        CurrentC=int(CurrentCinfor[0])
                        if int(methyinfor[0])<=CurrentC:
                            methyline=''
                            break
                        else:
                            CurrentCline=ChromeCfile.readline()
                            TCNum=TCNum+1            
            if len(MethydataSet)==SampleTotalNum:  #IF methyline is not null, calculate the entropy and pcc, out put them into file
                CommonCNum=CommonCNum+1
                for i in range(len(MethydataSet)):
                    MethydataSet[i]=MethydataSet[i]*100.0/MaxMethyValue
                if CNum==0:
                    PreviousMethydataSet=MethydataSet
                EucDistance=scipy.spatial.distance.euclidean(PreviousMethydataSet, MethydataSet)/100/SqrtSampleTotalNum  #calculate the euclidean distance between the methylation leveles of current C and previous C
                #PCC=scipy.stats.pearsonr(PreviousMethydataSet, MethydataSet)   #calculate the PCC between the methylation leveles of current C and previous C
                #DifferMethydataSet=(PreviousMethydataSet-MethydataSet+1)/2
                DifferMethydataSet=list(map(lambda x: abs(x[0]-x[1]), zip(PreviousMethydataSet, MethydataSet)))
                #print PreviousMethydataSet
                #print MethydataSet
                #print DifferMethydataSet
                PreviousMethydataSet=MethydataSet
                MethydataSet=tuple(MethydataSet)
                MethySpecificity=1.0-Methyentropy.EntropyCalculate(MethydataSet)
                DifferMethydataSet=tuple(DifferMethydataSet)
                DifferEntropyvalue=Methyentropy.EntropyCalculate(DifferMethydataSet)
                methyline=methyline+"\t"+str(MethySpecificity)+"\t"+str(DifferEntropyvalue)+"\t"+str(round(EucDistance,3))+"\n"
                MergedChromemethyfile.write(methyline)
                EntropyWig.write(str(CurrentC)+"\t"+str(MethySpecificity)+"\n")
                CNum+=1
        ChromeCfile.close()
        MergedChromemethyfile.close()
        for Sample in Samplelist: methyfilehandles.get(Sample,-1).close()
        EntropyWig.close()
        return TCNum,CommonCNum     

    def run(self,chromes,CFolderName,OutFolder,statisticout):
        Starttime=time.strftime('%Y-%m-%d %X',time.localtime(time.time()))
        print Starttime+" Start to calculate methylation specificiy ..."
        statisticout.write(Starttime+" Start to calculate methylation specificiy ...\n")
        MaxMethyValue=1    #The max methylation level,default is 1
        
        MethyDataFolderName=OutFolder+'SplitedMethy/'
        MergedmethyFolderName=OutFolder+'MethylationSpecificity/'
        if MergedmethyFolderName:
                # use a output directory to store merged Methylation data 
                if not os.path.exists( MergedmethyFolderName ):
                    try:
                        os.makedirs( MergedmethyFolderName )
                    except:
                        sys.exit( "Output directory (%s) could not be created. Terminating program." % MergedmethyFolderName )
        EntropyWig=gzip.open(MergedmethyFolderName+"MethylationSpecificity.wig.gz",'wb') #Biult a wig file 
        EntropyWig.write("track type=wiggle_0 name=\"Methylation Specificity of across Samples\" visibility=full color=20,150,20 altColor=150,20,20 windowingFunction=mean\n")
        EntropyWig.close()
        MethyMerge=MethyMergeEntropy()
        TotalCNum=0
        TotalCommonCNum=0
        for i in range(0,len(chromes)):
                ChromeCfileName=CFolderName+chromes[i]+".txt.gz"
                Chromedirpath=MethyDataFolderName+chromes[i]+"/*.wig.gz"
                Starttime=time.strftime('%Y-%m-%d %X',time.localtime(time.time()))
                print Starttime+" Calculating Methylation Specificity for "+chromes[i]
                statisticout.write(Starttime+" Calculating Methylation Specificity for "+chromes[i]+"\n")
                TCNum,CommonCNum=MethyMerge.MergefileBSSeqWigGznoRepEntropyCbase1Methybase1DiffEntropyED(chromes[i],ChromeCfileName,Chromedirpath,MergedmethyFolderName,MaxMethyValue,statisticout)
                TotalCNum=TotalCNum+TCNum
                TotalCommonCNum=TotalCommonCNum+CommonCNum
        return TotalCNum,TotalCommonCNum



if __name__ == '__main__':
    Starttime=time.strftime('%Y-%m-%d %A %X',time.localtime(time.time()))
    MaxMethyValue=1
    #RRBS
    chromes=['chr1','chr2','chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20','chr21','chr22','chrX','chrY','chrM']
    #chromes=['chr1','chr2']
    #MethyDataFolderName='J:/DNAMethylationData/ENCODE_RRBS/MergedRRBS/'
    #MergedmethyFolderName='J:/DNAMethylationData/ENCODE_RRBS/MergedRRBS/Mergedmethy/'
    #WaveletEntropyFolderName='J:/DNAMethylationData/ENCODE_RRBS/MergedRRBS/WaveletEntropy/'
    #BS-seq
    #chromes=['chr1','chr2','chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20','chr21','chr22','chrX','chrY']
    #chromes=['chrM','chrY']
    CFolderName='J:/HumanHg19Seq/ScanCresult/'
    MethyDataFolderName='J:/DNAMethylation/Human/BSSeq/RoadmapBSSeq/20140530_usedMerged_90percent/'
    MergedmethyFolderName=MethyDataFolderName+'MethylationSpecificity20140531/'
    statisticout=open(MergedmethyFolderName+"statistic.txt",'w')
    statisticout.write(Starttime+"\n")
    EntropyWig=gzip.open(MergedmethyFolderName+"MethylationSpecificity.wig.gz",'wb') #Biult a wig file 
    EntropyWig.write("track type=wiggle_0 name=\"Methylation Specificity of Human Tissues\" visibility=full color=20,150,20 altColor=150,20,20 windowingFunction=mean\n")
    EntropyWig.close()
    statisticout.close()
    #WaveletEntropyFolderName=MethyDataFolderName+'WaveletEntropy/'
    MethyMerge=MethyMergeEntropy()
    for i in range(0,len(chromes)):
            ChromeCfileName=CFolderName+chromes[i]+".txt.gz"
            Chromedirpath=MethyDataFolderName+chromes[i]+"/*.wig.gz"
            print chromes[i]
            MethyMerge.MergefileBSSeqWigGznoRepEntropyCbase1Methybase1DiffEntropyED(chromes[i],ChromeCfileName,Chromedirpath,MergedmethyFolderName,MaxMethyValue)
            #MethyMerge.Wavelet(chromes[i],MergedmethyFolderName,WaveletEntropyFolderName)
    Endtime=time.strftime('%Y-%m-%d %A %X',time.localtime(time.time()))
    statisticout=open(MergedmethyFolderName+"statistic.txt",'a')
    statisticout.write(Endtime+"\n")
    statisticout.close()



#!/usr/bin/env python
# Time-stamp: <2017-06-13 12:51:00 Hongbo Liu>

"""Description: SMART setup

Copyright (c) 2017 Hongbo Liu <hongbo919@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included
with the distribution).

@status: release candidate
@version: 2.1.8
@author:  Hongbo Liu
@contact: hongbo919@gmail.com
"""

# ------------------------------------
# python modules
#/Users/hongbo.liu/Documents/HongboData/Programming/Code/PyDev/Pythonworkspace/SMART
# Make pacage in Linux system
# python setup.py sdist bdist_wheel
# twine upload dist/SMART_BS_Seq-2.1.5-py2-none-any.whl dist/SMART-BS-Seq-2.1.5.tar.gz
# ------------------------------------
#from distutils.core import setup
from setuptools import setup
import sys
# Use build_ext from Cython if found
command_classes = {}

def main():
    if float(sys.version[:3])<2.7 or float(sys.version[:3])>=2.8:
        sys.stderr.write("CRITICAL: Python version must be 2.7!\n")
        sys.exit(1)
    
    setup(name="SMART-BS-Seq",
          version="2.1.8",
          description="Specific Methylation Analysis and Report Tool 2",
          long_description=open('README.rst', 'rt').read(),
          author='Hongbo Liu',
          author_email='hongbo919@gmail.com',
          url='http://fame.edbc.org/smart/',
          package_dir={'SMART' : 'SMART'},
          packages=['SMART'],
          package_data={'SMART':['Example/MethylMatrix_Test.txt','Example/CpGisland_hg19.bed']},
          #py_modules = ['Splitchrome','MethyMergeEntropy','SegmentationNormal','Folderprocess','NewEntropy','NewEntropyNormal'],    
          scripts=['bin/SMART'],
          classifiers=[
              'Development Status :: 5 - Production/Stable',
              'Programming Language :: Python',
              'License :: OSI Approved :: Python Software Foundation License',
              'Intended Audience :: End Users/Desktop',
              'Operating System :: POSIX :: Linux',
              'Operating System :: MacOS',
              ],
          install_requires=['scipy>=0.18.1','numpy>=1.11.2','statsmodels>=0.8.0','pandas>=0.19.2'],
          cmdclass = command_classes
          #install_requires=['scipy>=0.13',#],
          )


if __name__ == '__main__':
    main()



#!/usr/bin/env python
'''
Created on 2013.11.10

@author: liuhongbo
'''
import glob

class Folderprocess():
    def __init__(self):
        '''
        Constructor
        '''
    def readFolderfile(self,InFolderName):
        '''
        Read the file name in a folder
        '''
        #print "FileIn folder:"+InFolderName
        filenames = glob.glob(InFolderName)
        return filenames



if __name__ == '__main__':
    FolderName='J:/RoadmapBSSeq/20131216_used/*'
    FILENAMEOUT=open("J:/RoadmapBSSeq/UsedFileName.txt",'w')
    VisitFolder=Folderprocess()
    filenames=VisitFolder.readFolderfile(FolderName)
    print filenames
    for filename in filenames:
        FILENAMEOUT.write(filename+"\n")